<?php

/**
 * Classe per interagire con l'API REST di gCloud Storage
 *
 * @version 1.1.1
 * @package GCloud-SDK-PHP
 */
/**
 * Dipendiamo dalla HTTP_Request2 di PEAR
 */
require_once 'HTTP/Request2.php';

/**
 * Definizione della classe principale
 * 
 * @author Andrea Brancatelli <abrancatelli@schema31.it>
 * @package gCloud_Storage
 */
class gCloud_Storage {

    /**
     * Attuale versione della libreria
     *
     * @var string
     * @access public
     */
    const VERSION = "gCloud_Storage 1.1.1 [Pear]";

    /**
     * Repository su cui operare
     *
     * @var string 
     * @access public
     */
    var $repositoryName = "";

    /**
     * Chiave di autenticazione
     *
     * @var string 
     * @access public
     */
    var $Authentication = "";

    /**
     * gCloud Storage Base Path - hardcoded a "storage.gcloud.schema31.it"
     *
     * @var string 
     * @access public
     */
    var $gCloudStorageHost = "storage.gcloud.schema31.it";

    /**
     * LastError - l'ultimo errore ricevuto
     *
     * @var string 
     * @access public
     */
    var $LastError = "";
    private $putStatusCodeOK = array(201);
    private $postStatusCodeOK = array(201);
    private $getStatusCodeOK = array(200);
    private $deleteStatusCodeOK = array(200);

    /**
     * Carica un documento su gCloud Storage
     *
     * @param string $localFile Path del file da inviare su questo filesystem
     * @param string $mime Mime part del file da inviare (default application/octet-stream)
     * @param string $publicName Nome del file da inviare a gCloud Storage (di default prende il basename di $localFile)
     * @param string $fileKey Riferimento ad una fileKey già esistente per effettuare la "sovrascrittura" di un file
     * @return boolean|string torna la fileKey ritornata da gCloud Storage oppure FALSE in caso di errore
     * @access public
     */
    function sendFile($localFile, $mime = "application/octet-stream", $publicName = FALSE, $fileKey = FALSE) {

        /**
         * Se non ci viene passato un publicName esplicito lo ricaviamo dal nome del file da inviare
         */
        if ($publicName == FALSE) {
            $publicName = basename($localFile);
        }

        /**
         * Verifico se c'è una fileKey preimpostata
         */
        if ($fileKey != FALSE) {
            $extraUrl = "fileKey/" . $fileKey;
        } else {
            $extraUrl = "";
        }

        /**
         * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
         */
        $request = new HTTP_Request2('http://' . $this->gCloudStorageHost . "/api/resource/" . $extraUrl);
        $request->setMethod(HTTP_Request2::METHOD_POST)
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('User-Agent', self::VERSION)
                ->setHeader('Expect', "")
                ->addUpload('somefile', $localFile, $publicName, $mime);

        /**
         * Catturo la response della richiesta
         */
        $HTTPRequest2Response = $request->send();
        $responseBody = $HTTPRequest2Response->getBody();
        $responseStatus = $HTTPRequest2Response->getStatus();

        /**
         * Mi assicuro che mi sia stato tornato un oggetto e che non contenga "error".
         * Questa cosa va fatta più robusta.
         * Comunque intanto ritorno FALSE oppure il fileKey in base alle circostanze
         */
        if (!in_array($responseStatus, $this->postStatusCodeOK)) {
            $fileKey = FALSE;
            $this->LastError = $responseBody;
        } else {
            $response = json_decode($responseBody);
            if (!is_object($response)) {
                $fileKey = FALSE;
            } else {
                $fileKey = $response->Result[0]->fileKey;
            }
        }

        return $fileKey;
    }

    /**
     * Carica N documenti su gCloud Storage
     * 
     * @param array $localFiles Array contenente i file per cui effettuare l'upload
     * @param string $fileKey Riferimento ad una fileKey già esistente per effettuare la "sovrascrittura" di un file
     * @return boolean|string torna la fileKey ritornata da gCloud Storage oppure FALSE in caso di errore
     * @access public
     */
    function sendMultipleFiles($localFiles = array(), $fileKey = FALSE) {

        // Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
        $request = new HTTP_Request2('http://' . $this->gCloudStorageHost . "/api/resource/" . ($fileKey != FALSE ? "fileKey/" . $fileKey : ''));
        $request->setMethod(HTTP_Request2::METHOD_POST)
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('User-Agent', self::VERSION)
                ->setHeader('Expect', "");

        foreach ($localFiles as $index => $localFile) {

            // estrae il file path
            $localFilePath = (array_key_exists('path', $localFile) ? $localFile['path'] : '');

            // verifica che il file esista
            if (strlen($localFilePath) == 0 || !file_exists($localFilePath)) {
                $this->LastError = 'File not found: ' . $localFilePath;
                return FALSE;
            }

            // Se non ci viene passato un publicName esplicito lo ricaviamo dal nome del file da inviare
            $publicName = ( (array_key_exists('publicName', $localFile) && strlen($localFile['publicName']) > 0) ? $localFile['publicName'] : basename($localFilePath) );

            // Se non ci viene passato un mime type esplicito usiamo il default
            $mime = ( (array_key_exists('mime', $localFile) && strlen($localFile['mime']) > 0) ? $localFile['mime'] : 'application/octet-stream' );

            // Aggiunge all'oggetto HTTP_Request2 il file
            $request->addUpload("fileUpload_$index", $localFilePath, $publicName, $mime);
        }

        /**
         * Catturo la response della richiesta
         */
        $HTTPRequest2Response = $request->send();
        $responseBody = $HTTPRequest2Response->getBody();
        $responseStatus = $HTTPRequest2Response->getStatus();

        /**
         * Mi assicuro che mi sia stato tornato un oggetto e che non contenga "error".
         * Questa cosa va fatta più robusta.
         * Comunque intanto ritorno FALSE oppure il fileKey in base alle circostanze
         */
        if (!in_array($responseStatus, $this->postStatusCodeOK)) {
            $this->LastError = $responseBody;
            return FALSE;
        } else {
            $response = json_decode($responseBody);
            if (!is_object($response)) {
                return FALSE;
            } else {
                return $response->Result;
            }
        }
    }

    /**
     * Torna le informazioni su un dato file
     */
    function detailFile($fileKey, $fileVersion = FALSE, $chain = NULL, $subFile = NULL) {

        /**
         * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
         */
        $url = 'http://' . $this->gCloudStorageHost . '/api/resource/fileKey/' . $fileKey;

        // se non è stata specificata una versione specifica o è stato passato il carattere '_', usa l'ultima versione disponibile
        $url .= ( ($fileVersion != FALSE && $fileVersion != '_') ? "/fileVersion/$fileVersion" : '/_/' );

        // se si sta richedendo un file generato da una chain, è necessario indicare sia il nome della chain che l'indice di output del file
        $url .= ( ($chain != NULL && $subFile != NULL) ? "/chain/$chain/chainFileIndex/$subFile" : '' );

        $request = new HTTP_Request2($url);
        $request->setConfig('follow_redirects', TRUE);
        $request->setConfig('max_redirects', 20);
        $request->setConfig('strict_redirects', TRUE);
        $request->setMethod(HTTP_Request2::METHOD_GET)
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('User-Agent', self::VERSION);

        /**
         * Catturo la response della richiesta
         */
        $HTTPRequest2Response = $request->send();
        $responseBody = $HTTPRequest2Response->getBody();
        $responseStatus = $HTTPRequest2Response->getStatus();

        /**
         * Mi assicuro che mi sia stato tornato un oggetto e che non contenga "error".
         * Questa cosa va fatta più robusta.
         * Comunque intanto ritorno FALSE oppure il fileKey in base alle circostanze
         */
        if (!in_array($responseStatus, $this->getStatusCodeOK)) {
            $this->LastError = $responseBody;
            return FALSE;
        } else {
            $response = json_decode($responseBody);
            if (!is_object($response)) {
                return FALSE;
            } else {
                return $response;
            }
        }
    }

    /**
     * Scarica un documento da gCloud Storage
     */
    function getFile($fileKey, $fileVersion = FALSE, $chain = NULL, $subFile = NULL) {

        /*
         * anche in presenza degli attributi $chain e $subFile, vengono estratte le informazioni relative alla risorsa 'master' in quanto:
         * 1. la risorsa 'master' è utilizzata come riferimento anche per tutte le eventuali risorse generate da una chain
         * 2. la risorsa a cui si sta puntando potrebbe essere generata da una chain di tipo 'OUTPUT', quindi fisicamente non presente a sistema
         */
        $response = $this->detailFile($fileKey, $fileVersion);

        if ($response === FALSE) {
            // file non valido
            return FALSE;
        }

        /**
         * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
         */
        $url = $response->friendlyUrl;

        // aggiunge all'url i riferimenti alla chain (se necessario)
        $url .= ( ($chain != NULL && $subFile != NULL) ? "/$chain/$subFile" : '' );

        $request = new HTTP_Request2($url);
        $request->setConfig('follow_redirects', TRUE);
        $request->setConfig('max_redirects', 20);
        $request->setConfig('strict_redirects', TRUE);
        $request->setMethod(HTTP_Request2::METHOD_GET)
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('User-Agent', self::VERSION);

        // CONTROLLO ERRORE:
        $HTTPRequest2Response = $request->send();
        $responseBody = $HTTPRequest2Response->getBody();
        $responseStatus = $HTTPRequest2Response->getStatus();

        if (!in_array($responseStatus, $this->getStatusCodeOK)) {
            $this->LastError = $responseBody;
            return FALSE;
        } else {
            return $responseBody;
        }
    }

    /**
     * Cancella un documento da gCloud Storage
     */
    function deleteFile($fileKey, $fileVersion = FALSE) {

        /**
         * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
         */
        $url = 'http://' . $this->gCloudStorageHost . '/api/resource/fileKey/' . $fileKey;
        if ($fileVersion != FALSE) {
            $url .= '/fileVersion/' . $fileVersion;
        }

        $request = new HTTP_Request2($url);

        $request->setConfig('follow_redirects', TRUE);
        $request->setConfig('max_redirects', 20);
        $request->setConfig('strict_redirects', TRUE);
        $request->setMethod(HTTP_Request2::METHOD_DELETE)
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('User-Agent', self::VERSION);

        /**
         * Catturo la response della richiesta
         */
        $HTTPRequest2Response = $request->send();
        $responseBody = $HTTPRequest2Response->getBody();
        $responseStatus = $HTTPRequest2Response->getStatus();

        /**
         * Mi assicuro che mi sia stato tornato un oggetto e che non contenga "error".
         * Questa cosa va fatta pi<9d> robusta.
         * Comunque intanto ritorno FALSE oppure il fileKey in base alle circostanze
         */
        if (!in_array($responseStatus, $this->deleteStatusCodeOK)) {
            //$fileKey = FALSE;
            $this->LastError = $responseBody;
            return FALSE;
        } else {
            $response = json_decode($responseBody);
            if (!is_object($response)) {
                return FALSE;
            } else {
                return $response;
            }
        }
    }

    /**
     * Ritorna lo short-url per accedere ad una risorsa memorizzata all'interno di un repository privato
     * @param string $fileKey
     * @param int $fileVersion (facoltativo - se omesso ritorna l'ultima versione per la fileKey passata come parametro)
     * @param int $expiresAt (facoltativo - indica il numero di secondi per cui il link generato sarà valido)
     * @param string $whitelist (facoltativo - lista di indirizzi IP, separati da punto e virgola, abilitati ad accedere al link generato )
     * @return boolean|string url della risorsa oppure FALSE in caso di errore
     * @access public
     */
    function getShortUrl($fileKey, $fileVersion = NULL, $expiresAt = NULL, $whitelist = NULL) {

        // Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
        $request = new HTTP_Request2('http://' . $this->gCloudStorageHost . "/api/shorturl");
        $request->setMethod(HTTP_Request2::METHOD_POST)
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('fileKey', $fileKey)
                ->setHeader('User-Agent', self::VERSION)
                ->setHeader('Expect', "");

        // è stata indicata una fileVersion?
        if (!is_null($fileVersion)) {
            $request->setHeader('fileVersion', $fileVersion);
        }

        // è stata indicata una durata massima per la validità dell'url?
        if (!is_null($expiresAt)) {
            $request->setHeader('expiresAt', $expiresAt);
        }

        // è stata indicata una white-list di indirizzi IP, i quali potranno accedere alla risorsa?
        if (!is_null($whitelist)) {
            $request->setHeader('whitelist', $whitelist);
        }

        // Catturo la response della richiesta
        $HTTPRequest2Response = $request->send();
        $responseBody = $HTTPRequest2Response->getBody();
        $responseStatus = $HTTPRequest2Response->getStatus();

        /**
         * Mi assicuro che mi sia stato tornato un oggetto e che non contenga "error".
         * Questa cosa va fatta più robusta.
         * Comunque intanto ritorno FALSE oppure il fileKey in base alle circostanze
         */
        if (!in_array($responseStatus, $this->postStatusCodeOK)) {
            $this->LastError = $responseBody;
            return FALSE;
        } else {
            $response = json_decode($responseBody);

            if (!is_object($response)) {
                return FALSE;
            } else {
                return $response->shortUrl;
            }
        }
    }

}
